classdef GSIPProblem<hgsetget
    % GSIPProblem is an abstract class. This class is a parent class for
    % all implemented GSIP-problems. It contains the basic properties and
    % the basic functions of every GSIP-problem.
    
    
    % class for the formulation of the problems
    % x are the upper level variables, x allways consists of a 1xm vector.
    % y are the lower level variables, y constists of a kxn vector where k
    % is variable.
    % i is a integer which gives the number of the semi-infinite
    % constraint.
    % j is a integer which gives the number of the lower level.
    properties (Abstract, GetAccess=protected)
        %Upper Level
        
        nrOfUppLevVars          %number of parameters x
        nrOfSemiInfConstrs      %number of semi-infinite constraints
        
        boundsOnUppLevVars      %bounds of parameters x
        
        initValsOfUppLevVars    %initial values of the parameters x
        
        nrOfNonlinIneqsUppLev   %number of non linear and non semi-infinite inequality constraints on x
        nrOfNonlinEqsUppLev     %number of non linear and non semi-infinite equality constraints on x              
                          
        linIneqsOnUppLevVars    %linear inequality constraints on x
        linEqsOnUppLevVars      %lineq equality constraints on x
        
        %Lower Level
        
        nrOfLowLevVars          %dimension of the index variables y
        
        %problem assignment
        
        nrOfIndexset            %nr of different index sets
        con2LowLev              %problem assignment as an array. i-th entry 
                                %is the index of the index set
                                %corresponding to the i-th semi-infinite
                                %constraint
       
        
    end
    
   
     methods (Abstract)
         
        %Upper-Level
        
        %f=evalObjFun(x) evaluates the objective function f for the point x
        f=evalObjFun(obj,x)                       
        
        %g=evalSemiInfConstrs(i,x,y) evaluates teh i-th semi infinite
        %constraint g_i for x and y. 
        g=evalSemiInfConstrs(obj,i,x,y)        
   
                
        %y=evalTrafoFun(j,x,z) evaluates the transformation function t for
        %the j-th indexset at the points x and y
        y=evalTrafoFun(obj,j,x,z)
        
        %z=evalInvTrafoFun(j,x,y) returns a preimage of the point y under 
        %the transformation function t for the j-th indexset and a fixed
        %point x.
        z=evalInvTrafoFun(obj,j,x,y)
        
        
       
     end
    methods
        
        
        function grad_f=gradObjFun(obj,x)
            %grad_f=gradObjFun(x) returns the gradient of the objective
            %function f at the point x. If the function is not implemented,
            %the gradient is computed numerically.
            grad_f=obj.DxObjFun(x)';
        end
        
        function Dxf=DxObjFun(obj,x0)
            %Dxf=DxObjFun(x) returns the Jacobian matrix of the objective
            %function f at the point x. If the function is not implemented,
            %the derivative is computed numerically.
           Dxf=getDerivativeViaDifferenceQuotient(@(x) obj.evalObjFun(x),x0,{1});            
        end
        
        function DxDxf=DxDxObjFun(obj,x0)
            %DxDxf=DxDxObjFun(x) returns the Hessian matrix of the
            %objective function f at the point x. If the funciton is not
            %implemented, the Hessian is computed numerically.
           DxDxf=getSecDerivativeViaDifferenceQuotient(@(x) obj.evalObjFun(x),x0,{[]});            
        end
        
        %Derivative semi-infinite constraints
        function Dy_g=DySemiInfConstrs(obj,i,x,y0)
            %Dy_g=DySemiInfConstrs(i,x,y) returns the derivative (as
            %Jacobian) of the i-th semi infinite constraint with respect of
            %y. Thus, the point x is fixed. If the function is not
            %implemented, a numerical solution will be computed.
            Dy_g=zeros(size(y0,1),size(y0,2));
            for iPoint=1:size(y0,1)
                Dy_g(iPoint,:)=getDerivativeViaDifferenceQuotient(@(y) obj.evalSemiInfConstrs(i,x,y),y0(iPoint,:),{1});
            end
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x0,y)
            %Dx_g=DxSemiInfConstrs(i,x,y) returns the derivative (as
            %Jacobian) of the i-th semi infinite constraint with respect of
            %x. Thus, the point y is fixed. If the function is not
            %implemented, a numerical solution will be computed.
            Dx_g=zeros(size(y,1),size(x0,2));
            for iPoint=1:size(y,1)
                Dx_g(iPoint,:)=getDerivativeViaDifferenceQuotient(@(x) obj.evalSemiInfConstrs(i,x,y(iPoint,:)),x0,{1});
            end
        end
        function DxDx_g=DxDxSemiInfConstrs(obj,i,x0,y)
            %DxDx_g=DxDxSemiInfConstrs(i,x,y) returns the seccond
            %derivative with respect to x of the i-th semi infinite
            %constraint. The point y is fixed. If the function is not
            %implemented, a numerical solution will be computed.
            DxDx_g=zeros(size(x0,2),size(x0,2),size(y,1));
            for iPoint=1:size(y,1)
                DxDx_g(:,:,iPoint)=getSecDerivativeViaDifferenceQuotient(@(x) obj.evalSemiInfConstrs(i,x,y(iPoint,:)),x0,{[]});
            end
        end
        function DxDy_g=DxDySemiInfConstrs(obj,i,x0,y0)
            %DxDy_g=DxDySemiInfConstrs(i,x,y) returns the mixed seccond
            %derivative with respect to y and x of the i-th semi infinite
            %constraint. If the function is not implemented, a numerical 
            %solution will be computed.
            DxDy_g=zeros(size(y0,2),size(x0,2),size(y0,1));
            for iPoint=1:size(y0,1)
                DxDy_g(:,:,iPoint)=getMixedSecDerivativeViaDifferenceQuotient(@(y,x) obj.evalSemiInfConstrs(i,x,y),y0(iPoint,:),x0,{[]});
            end
        end
        
        function DyDy_g=DyDySemiInfConstrs(obj,i,x,y0)
            %DyDy_g=DyDySemiInfConstrs(i,x,y) returns the seccond
            %derivative with respect to y of the i-th semi infinite
            %constraint. The point x is fixed. If the function is not
            %implemented, a numerical solution will be computed.
            DyDy_g=zeros(size(y0,2),size(y0,2),size(y0,1));
            for iPoint=1:size(y0,1)
                DyDy_g(:,:,iPoint)=getSecDerivativeViaDifferenceQuotient(@(y) obj.evalSemiInfConstrs(i,x,y),y0(iPoint,:),{[]});
            end
        end
        
        %Derivatives transformation 
        function Dz_y=DzTrafoFun(obj,j,x,z0)
            %Dz_y=DzTrafoFun(j,x,z) returns the derivative (as Jacobian) of
            %the transformation function t corresponding to the j-th index 
            %set with respect of z. The point x is fixed. If the funciton
            %is not implemented, a numerical solution will be computed.
            Dz_y=zeros(obj.nrOfLowLevVars(j),size(z0,2),size(z0,1));
            for iPoint=1:size(z0,1)
                Dz_y(:,:,iPoint)=getDerivativeViaDifferenceQuotient(@(z) obj.evalTrafoFun(j,x,z),z0(iPoint,:),{obj.nrOfLowLevVars(j)});
            end
        end
        function Dx_y=DxTrafoFun(obj,j,x0,z)
            %Dx_y=DxTrafoFun(j,x,z) returns the derivative (as Jacobian) of
            %the transformation function t corresponding to the j-th index 
            %set with respect of x. The point z is fixed. If the funciton
            %is not implemented, a numerical solution will be computed.
            Dx_y=zeros(obj.nrOfLowLevVars(j),size(x0,2),size(z,1));
            for iPoint=1:size(z,1)
                Dx_y(:,:,iPoint)=getDerivativeViaDifferenceQuotient(@(x) obj.evalTrafoFun(j,x,z(iPoint,:)),x0,{obj.nrOfLowLevVars(j)});
            end
        end
        
        function DxDx_y=DxDxTrafoFun(obj,j,x0,z)
            %DxDx_y=DxDxTrafoFun(j,x,z) returns the seccond
            %derivative with respect to x of the transformation function
            %corresponding to the j-th index set. The point z is fixed. If 
            %the function is not implemented, a numerical solution will be 
            %computed.
            DxDx_y=zeros(obj.nrOfLowLevVars(j),size(x0,2),size(x0,2),size(z,1));
            for iPoint=1:size(z,1)
                DxDx_y(:,:,:,iPoint)=getSecDerivativeViaDifferenceQuotient(@(x) obj.evalTrafoFun(j,x,z(iPoint,:)),x0,{[obj.nrOfLowLevVars(j)]});
            end
        end
        
        function DxDz_y=DxDzTrafoFun(obj,j,x0,z0)
            %DxDz_g=DxDzTrafoFun(j,x,z) returns the mixed seccond
            %derivative with respect to z and x of the transformation 
            %function corresponding to the j-th index set. If the function 
            %is not implemented, a numerical solution will be computed.
            DxDz_y=zeros(obj.nrOfLowLevVars(j),size(z0,2),size(x0,2),size(z0,1));
            for iPoint=1:size(z0,1)
                DxDz_y(:,:,:,iPoint)=getMixedSecDerivativeViaDifferenceQuotient(@(z,x) obj.evalTrafoFun(j,x,z),z0(iPoint,:),x0,{[obj.nrOfLowLevVars(j)]});                     
            end
        end
        
        function DzDz_y=DzDzTrafoFun(obj,j,x,z0)
            %DzDz_y=DzDzTrafoFun(j,x,z) returns the seccond
            %derivative with respect to z of the transformation function
            %corresponding to the j-th index set. The point x is fixed. If 
            %the function is not implemented, a numerical solution will be 
            %computed.
            DzDz_y=zeros(obj.nrOfLowLevVars(j),size(z0,2),size(z0,2),size(z0,1));
            for iPoint=1:size(z0,1)            
                DzDz_y(:,:,:,iPoint)=getSecDerivativeViaDifferenceQuotient(@(z) obj.evalTrafoFun(j,x,z),z0(iPoint,:),{[obj.nrOfLowLevVars(j)]});
            end
        end
        
        %nonlinear constraints
        function g=evalUppLevNonlinIneqConstrs(obj,x)
            %g=evalUppLevNonlinIneqConstrs(x) evaluates the non linear and
            %non semi-infinite inequality constraints on the parameters x. 
            %The output g is a vector. Every entry corresponds to one 
            %constraint. If the function is not implemented, a empty vector 
            %of size 1x0 is returned.
            g=zeros(1,0);
        end
        function h=evalUppLevNonlinEqConstrs(obj,x)
            %h=evalUppLevNonlinEqConstrs(x) evaluates the non linear and
            %non semi-infinite equality constraints on the parameters x. 
            %The output h is a vector. Every entry corresponds to one 
            %constraint. If the function is not implemented, a empty vector 
            %of size 1x0 is returned.
            h=zeros(1,0);
        end
         %Derivatives
        function  Dx_k=DxUppLevNonlinIneqConstrs(obj,x0)
            %Dx_k=DxUppLevNonlinIneqConstrs(x) returns the derivative (as 
            %Jacobian) of the non linear and non semi-infinite inequality
            %constraints on the parameters x. If the function is not 
            %implemented, a numerical solution will be computed.
            Dx_k=getDerivativeViaDifferenceQuotient(@(x) obj.evalUppLevNonlinIneqConstrs(x),x0,{obj.nrOfNonlinIneqsUppLev});
        end
        
        function Dx_k=DxUppLevNonlinEqConstrs(obj,x0)
            %Dx_k=DxUppLevNonlinEqConstrs(x) returns the derivative (as 
            %Jacobian) of the non linear and non semi-infinite equality 
            %constraints on the parameters x. If the function is not 
            %implemented, a numerical solution will be computed.
            Dx_k=getDerivativeViaDifferenceQuotient(@(x) obj.evalUppLevNonlinEqConstrs(x),x0,{obj.nrOfNonlinEqsUppLev});
        end
        
        function  DxDx_k=DxDxUppLevNonlinIneqConstrs(obj,x0)
            %DxDx_k=DxDxUppLevNonlinIneqConstrs(x) returns the seccond
            %derivative of the the non linear and non semi-infinite
            %inequality constraints on the parameters x. If the function is
            %not implemented, a numerical solution will be computed. For
            %the output size have a look into the documentation
        
            DxDx_k=getSecDerivativeViaDifferenceQuotient(@(x) obj.evalUppLevNonlinIneqConstrs(x),x0,{[obj.nrOfNonlinIneqsUppLev]});
        end
        
        function DxDx_k=DxDxUppLevNonlinEqConstrs(obj,x0)
            %DxDx_k=DxDxUppLevNonlinIneqConstrs(x) returns the seccond
            %derivative of the the non linear and non semi-infinite equality 
            %constraints on the parameters x. If the function is not 
            %implemented, a numerical solution will be computed. For
            %the output size have a look into the documentation
            DxDx_k=getSecDerivativeViaDifferenceQuotient(@(x) obj.evalUppLevNonlinEqConstrs(x),x0,{[obj.nrOfNonlinEqsUppLev]});
        end
    end
    
     
        %get-functions
    methods 
        %UpperLevel
        
        
        function [lb,ub]=getBoundsOnUppLevVars(obj)
            %[lb,ub]=getBoundsOnUppLevVars() returns the lower and the
            %upper bounds on the parameters x. lb and ub are two row
            %vectors. If a parameter is unbounded -inf/inf is returned.
            lb=obj.boundsOnUppLevVars(1,:);
            ub=obj.boundsOnUppLevVars(2,:);
        end
        
        % linear restrictions on x 
        function [A,b]=getLinEqsOnUppLevVars(obj)
            %[C,d]=getLinEqsOnUppLevVars() returns the linear equality 
            %constriants C*x= d.
            n=obj.nrOfUppLevVars;
            if size(obj.linEqsOnUppLevVars) ~= 0
               A=obj.linEqsOnUppLevVars(:,1:n);
               b=obj.linEqsOnUppLevVars(:,n+1);
            else
                n=obj.nrOfUppLevVars;
                A=zeros(0,n);
                b=zeros(0,1);
            end          
        end
        
        function [A,b]=getLinIneqsOnUppLevVars(obj)
            %[A,b]=getLinIneqsOnUppLevVars() returns the linear inequality 
            %constriants A*x<= b.
            
            n=obj.nrOfUppLevVars;
            if size(obj.linIneqsOnUppLevVars) ~= 0
               
               A=obj.linIneqsOnUppLevVars(:,1:n);
               b=obj.linIneqsOnUppLevVars(:,n+1);
            else
                A=zeros(0,n);
                b=zeros(0,1);
            end      
        end
        %number of nonlinear restrictions
        function n=getNrUppLevNonlinIneqs(obj)
            %n=getNrUppLevNonlinIneqs() returns the number of non linear
            %and non semi-infinite inequality constraints on x.
            n=obj.nrOfNonlinIneqsUppLev;                 
        end  
        
        function n=getNrUppLevNonlinEqs(obj)
            %n=getNrUppLevNonlinEqs() returns the number of non linear
            %and non semi-infinite equality constraints on x.
            n=obj.nrOfNonlinEqsUppLev;                 
        end 
        
        % number of variables and constraints of the upper level problem.
        function n=getNrUppLevVars(obj)
            %n=getNrUppLevVars() returns the number of parameters x.
            n=obj.nrOfUppLevVars;                   
        end
 
        function n=getNrSemiInfConstrs(obj)
            %n=getNrSemiInfConstrs() returns the number of semi-infinite
            %constraints
            n=obj.nrOfSemiInfConstrs;                   
        end 
        % initial values for the upper level variables
        function x0=getInitValsUppLevVars(obj)
            %x0=getInitValsUppLevVars() returns initial values of x. These
            %can be used as a starting point in an algorithm.
           x0=obj.initValsOfUppLevVars; 
        end
        
        function setInitValsUppLevVars(obj,x0)
            %setInitValsUppLevVars(x0) sets the initial values of the
            %parameters x.
            [sr,sc]=size(x0);
            assert((sr==1)&&(sc==obj.nrOfUppLevVars),'The new initial values must have the size 1xm')
           obj.initValsOfUppLevVars=x0; 
        end
        
        
        %LowerLevel
        function n=getNrLowLevVars(obj,j)
            %n=getNrLowLevVars(obj,j) returns the dimension of the variable
            %y.
            n=obj.nrOfLowLevVars(j);                   
        end
 
        
        %problem assignment    
        function n=getNrIndexSets(obj)
            %n=getNrIndexSets() returns the number of different index sets
            n=obj.nrOfIndexset;
        end
        function k=getIndexSetOfSemiInfConstr(obj,i)
            %k=getIndexSetOfSemiInfConstr(i) returns the index of the index
            %set that belongs to the i-th semi-infinite constraint.
            k=obj.con2LowLev(i);
        end
        
         
       
            
    end
    
    
    
end

