classdef Design <hgsetget
    %Design is an abstract class. This class is the super class for all
    %designs. It contains some get functions and standard derivative
    %functions.
    
    properties (Abstract, GetAccess= protected)
        
        dim
        
        nrOfParams                              
        nrOfDesignConstrs                        
        initValsParams                      
        
        linEqParamConstrs                        
        linIneqParamConstrs                      
        boundsOnParams
        
        nrOfNonlinParamEqConstrs                 
        nrOfNonlinParamIneqConstrs               

       
    end
    
    methods (Abstract)
        %v=vol(x) returns the volume of the design.         
        v=vol(obj,x)                                               
        
        %y=getInteriorPoint(x) returns a point that lies in the interior of
        %the design.
        y=getInteriorPoint(obj,x)                    
                               
        
   
    end
    
    methods
        %Derivatives
        function grad_v=gradVol(obj,x0)
            %grad_v=gradVol(x) returns the gradient of the volume at the
            %point x. If the function is not implemented, the gradient is 
            %computed numerically.
            grad_v=obj.DxVol(x0)';
        end
        function DxVol=DxVol(obj,x0)
            %DxVol=DxVol(x) returns the Jacobian matrix of the volume
            %function at the point x. If the function is not implemented,
            %the derivative is computed numerically.
           DxVol=getDerivativeViaDifferenceQuotient(@(x) obj.vol(x),x0,{1});            
        end
        function DxDxVol=DxDxVol(obj,x0)
            %DxDxVol=DxDxVol(x) returns the Hessian matrix of the
            %volume function at the point x. If the function is not
            %implemented, the Hessian is computed numerically.
            DxDxVol=getSecDerivativeViaDifferenceQuotient(@(x) obj.vol(x),x0,{[]});
           %DxDxVol=getDerivativeViaDifferenceQuotient(@(x) obj.DxVol(x),x0,{obj.nrOfParams});            
        end
        
        %u   
        function u=evalNonlinDescrFuns(obj,x,y)
            %u=evalNonlinDescrFuns(x,y) evaluates the non linear 
            %describing functions. The output u is a vector where every 
            %entry corresponds to one constraint. If no such constraints 
            %are implemented, a empty vector of size 1x0 is returned.
                u=zeros(1,0);
        end
        function Dy_u=DyNonlinDescrFuns(obj,x,y0)
            %Dy_u=DyNonlinDescrFuns(x,y) returns the derivative (As 
            %Jacobian) of the non linear describing functions with respect 
            %to y. If no function is implemented a numerical solution will
            %be computed.
            nrLowLevConstr=size(obj.evalNonlinDescrFuns(x,y0),2);
            Dy_u=getDerivativeViaDifferenceQuotient(@(y) obj.evalNonlinDescrFuns(x,y),y0,{nrLowLevConstr});
        end
        function Dx_u=DxNonlinDescrFuns(obj,x0,y)
            %Dx_u=DxNonlinDescrFuns(x,y) returns the derivative (As 
            %Jacobian) of the non linear describing functions with respect 
            %to x. If no function is implemented, a numerical solution will
            %be computed.
            nrLowLevConstr=size(obj.evalNonlinDescrFuns(x0,y),2);
            Dx_u=getDerivativeViaDifferenceQuotient(@(x) obj.evalNonlinDescrFuns(x,y),x0,{nrLowLevConstr});
        end
        
        function DxDx_u=DxDxNonlinDescrFuns(obj,x0,y)
            %DxDx_u=DxDxNonlinDescrFuns(x,y) returns the seccond derivative
            %of the non linear describing functions with respect to x. If 
            %no function is implemented, a numerical solution will be
            %computed. For the exact output size have a look into the
            %documentation.
            nrLowLevConstr=size(obj.evalNonlinDescrFuns(x0,y),2);
            DxDx_u=getSecDerivativeViaDifferenceQuotient(@(x) obj.evalNonlinDescrFuns(x,y),x0,{[nrLowLevConstr]});                     
        end
        
        function DxDy_u=DxDyNonlinDescrFuns(obj,x0,y0)
            %DxDy_u=DxDyNonlinDescrFuns(x,y) returns the mixed seccond
            %derivative of the non linear describing functions with respect
            %to x and y. If no function is implemented, a numerical 
            %solution will be computed. For the exact output size have a
            %look into the documentation.
            nrLowLevConstr=size(obj.evalNonlinDescrFuns(x0,y0),2);
            DxDy_u=getMixedSecDerivativeViaDifferenceQuotient(@(y,x) obj.evalNonlinDescrFuns(x,y),y0,x0,{[nrLowLevConstr]});                     
        end
        
        function DyDy_u=DyDyNonlinDescrFuns(obj,x,y0)
            %DyDy_u=DyDyNonlinDescrFuns(x,y) returns the seccond derivative
            %of the non linear describing functions with respect to y. If
            %no function is implemented, a numerical solution will be
            %computed. For the exact output size have a look into the
            %documentation.
            nrLowLevConstr=size(obj.evalNonlinDescrFuns(x,y0),2);
            DyDy_u=getSecDerivativeViaDifferenceQuotient(@(y) obj.evalNonlinDescrFuns(x,y),y0,{[nrLowLevConstr]});
        end
        
        
        %transformation function
        function y=evalTrafoFun(obj,x,z)
            %y=evalTrafoFun(x,z) evaluates the transformation function of 
            %the design at the points x and y
            y=z;
        end
        
        function z=evalInvTrafoFun(obj,x,y)    
            %z=evalInvTrafoFun(x,y) returns a preimage of the point y under 
            %the transformation function t for fixed parameters x.
            z=y;
        end
        
        
        
        
        function Dz_y=DzTrafoFun(obj,x,z0)
            %Dz_y=DzTrafoFun(j,z) returns the derivative (as Jacobian) of
            %the transformation function t with respect of z. The point x 
            %is fixed. If the funciton is not implemented, a numerical 
            %solution will be computed.
            Dz_y=getDerivativeViaDifferenceQuotient(@(z) obj.evalTrafoFun(x,z),z0,{obj.dim});
        end
        function Dx_y=DxTrafoFun(obj,x0,z)
            %Dx_y=DxTrafoFun(x,z) returns the derivative (as Jacobian) of
            %the transformation function t with respect of x. The point z 
            %is fixed. If the funciton is not implemented, a numerical
            %solution will be computed.
            Dx_y=getDerivativeViaDifferenceQuotient(@(x) obj.evalTrafoFun(x,z),x0,{obj.dim});
        end
        
        function DxDx_y=DxDxTrafoFun(obj,x0,z)
            %DxDx_y=DxDxTrafoFun(j,x,z) returns the seccond
            %derivative with respect to x of the transformation function. 
            %The point z is fixed. If the function is not implemented, a 
            %numerical solution will be computed.
            DxDx_y=getSecDerivativeViaDifferenceQuotient(@(x) obj.evalTrafoFun(x,z),x0,{[obj.dim]});                     
        end
        
        function DxDz_y=DxDzTrafoFun(obj,x0,z0)
            %DxDz_g=DDxDzTrafoFun(j,x,z) returns the mixed seccond
            %derivative with respect to z and x of the transformation 
            %function. If the function is not implemented, a numerical
            %solution will be computed.
            DxDz_y=getMixedSecDerivativeViaDifferenceQuotient(@(z,x) obj.evalTrafoFun(x,z),z0,x0,{[obj.dim]});                     
        end
        
        function DzDz_y=DzDzTrafoFun(obj,x,z0)
            %DzDz_y=DzDzTrafoFun(j,x,z) returns the seccond
            %derivative with respect to z of the transformation function.
            %The point x is fixed. If the function is not implemented, a 
            %numerical solution will be computed.
            DzDz_y=getSecDerivativeViaDifferenceQuotient(@(z) obj.evalTrafoFun(x,z),z0,{[obj.dim]});
        end
        
        
        
        
        
        % design functions
        
        function [A,b]=getLinDescrFuns(obj,x)
            %[A,b]=getLinDescrFuns(x) returns the linear describing
            %functions A*y<=b.
            A=zeros(0,obj.dim);
            b=zeros(0,1);
        end
        %Derivativs
        function [DxA,Dxb]=DxLinDescrFuns(obj,x0)
            %[DxA,Dxb]=DxLinDescrFuns(x) returns the derivative with
            %respect to x of the function [A,b]=getLinDescrFuns(x). If no
            %function is implemented, a numerical solution will be computed.
            %Dxb is the Jacobian. DxA is a 3-dimensional matrix where the
            %first matrix is the derivative with respect to x1 the seccond
            %with respect to x2 and so on.
            nrLinIneq=size(obj.getLinDescrFuns(x0),1);
            [DxA,Dxb]=getDerivativeViaDifferenceQuotient(@(x) obj.getLinDescrFuns(x),x0,{[nrLinIneq,obj.dim],[nrLinIneq,1]});
        end
        function [DxDxA,DxDxb]=DxDxLinDescrFuns(obj,x0)
            %[DxDxA,DxDxb]=DxDxLinDescrFuns(x) returns the seccond 
            %derivative with respect to x of the function
            %[A,b]=getLinDescrFuns(x). If no function is implemented, a
            %numerical solution will be computed. For the exact output size
            %have a look into the documentation.
            nrLinIneq=size(obj.getLinDescrFuns(x0),1);
            [DxDxA,DxDxb]=getSecDerivativeViaDifferenceQuotient(@(x) obj.getLinDescrFuns(x),x0,{[nrLinIneq,obj.dim],[nrLinIneq,1]});            
        end        
        %Derivativs
        function [lb,ub]=getBoundingBox(obj,x)
            %[lb,ub]=getBoundingBox(x) returns a bounding box of the design
            %fixed point x. lb and ub are rowvectors. If one variable yi is
            %unbounded, the bounds are -inf/inf.
            lb=-inf*ones(1,obj.dim);
            ub=inf*ones(1,obj.dim);
        end
        function [Dxlb,Dxub]=DxBoundingBox(obj,x0)
            %[Dxlb,Dxub]=DxBoundingBox(x) returns the derivative with
            %respect to x (as Jacobian) of the function 
            %[lb,ub]=getBoundingBox(x). If no function is implemented, a
            %numerical solution will be computed.
            [Dxlb,Dxub]=getDerivativeViaDifferenceQuotient(@(x) obj.getBoundingBox(x),x0,{obj.dim,obj.dim});
        end
        function [DxDxlb,DxDxub]=DxDxBoundingBox(obj,x0)
            %[DxDxlb,DxDxub]=DxDxBoundingBox(x) returns the seccond
            %derivative with respect to x of the function 
            %[lb,ub]=getBoundingBox(x). If no function is implemented, a
            %numerical solution will be computed. For the exact output size
            %have a look into the documentation.
            [DxDxlb,DxDxub]=getSecDerivativeViaDifferenceQuotient(@(x) obj.getBoundingBox(x),x0,{[obj.dim],[obj.dim]});
        end
        
       
        
        
        
        %constraints on parameters
        function k=evalNonlinEqConstrsOnParams(~,x)
            %h=evalNonlinEqConstrsOnParams(x) evaluates the non linear and
            %non semi-infinite equality constraints on the parameters x. 
            %The output h is a vector. Every entry corresponds to one 
            %constraint. If the function is not implemented, a empty vector 
            %of size 1x0 is returned.
            k=zeros(1,0);
        end
        function k=evalNonlinIneqConstrsOnParams(~,x)
            %g=evalNonlinIneqConstrsOnParams(x) evaluates the non linear
            %and non semi-infinite inequality constraints on the parameters
            %x. The output g is a vector. Every entry corresponds to one 
            %constraint. If the function is not implemented, a empty vector 
            %of size 1x0 is returned. 
            k=zeros(1,0);
        end
        %Derivatives
        function  Dx_k=DxNonlinEqConstrsOnParams(obj,x0)
            %Dx_k=DxNonlinEqConstrsOnParams(x) returns the derivative (as 
            %Jacobian) of the non linear and non semi-infinite equality 
            %constraints on the parameters x. If the function is not 
            %implemented, a numerical solution will be computed.
            Dx_k=getDerivativeViaDifferenceQuotient(@(x) obj.evalNonlinEqConstrsOnParams(x),x0,{obj.nrOfNonlinParamEqConstrs});
        end
        
        function Dx_k=DxNonlinIneqConstrsOnParams(obj,x0)
            %Dx_k=DxNonlinIneqConstrsOnParams(x) returns the derivative (as 
            %Jacobian) of the non linear and non semi-infinite inequality
            %constraints on the parameters x. If the function is not 
            %implemented, a numerical solution will be computed.
            Dx_k=getDerivativeViaDifferenceQuotient(@(x) obj.evalNonlinIneqConstrsOnParams(x),x0,{obj.nrOfNonlinParamIneqConstrs});
        end
        
        function  DxDx_k=DxDxNonlinEqConstrsOnParams(obj,x0)
            %DxDx_k=DxDxNonlinEqConstrsOnParams(x) returns the seccond
            %derivative of the the non linear and non semi-infinite
            %inequality constraints on the parameters x. If the function is
            %not implemented, a numerical solution will be computed. For
            %the output size have a look into the documentation.
            DxDx_k=getSecDerivativeViaDifferenceQuotient(@(x) obj.evalNonlinEqConstrsOnParams(x),x0,{[obj.nrOfNonlinParamEqConstrs]});
        end
        
        function DxDx_k=DxDxNonlinIneqConstrsOnParams(obj,x0)
            %DxDx_k=DxDxUppLevNonlinIneqConstrs(x) returns the seccond
            %derivative of the the non linear and non semi-infinite
            %inequality constraints on the parameters x. If the function is
            %not implemented, a numerical solution will be computed. For
            %the output size have a look into the documentation.
            DxDx_k=getSecDerivativeViaDifferenceQuotient(@(x) obj.evalNonlinIneqConstrsOnParams(x),x0,{[obj.nrOfNonlinParamIneqConstrs]});
        end
        
    end
    
    
    
    methods
        
        %set-functions
        function []=setInitValsParams (obj,x)
            %setInitValsParams(x0) set the initial values of the parameters
            %x to x0
            
            assert(obj.nrOfParams==max(size(x)),'initVals has to be a vector with size nrOfParams.');
            assert(1==min(size(x)),'initVals has to be a vector with size nrOfParams.')
            if size(x,1)==1
            else
                x=x';
            end
            obj.initValsParams=x;
            
        end
        %get-functions
        function d=getDim (obj)
            %d=getDim() returns the dimension of the design
            d=obj.dim;
        end
        function n=getNrParams (obj)
            %n=getNrParams() returns the number of parameters x, that
            %describe the design.
            n=obj.nrOfParams;
        end
        function n=getNrNonlinDescrFuns (obj)
            %n=getNrNonlinDescrFuns() returns the number of non linear
            %describing functions of the design.
            n=obj.nrOfDesignConstrs;
        end
        function x0=getInitValsParams (obj)
            %x0=getInitValsParams() returns initial parameters x0 for the
            %variables x.
            x0=obj.initValsParams;
        end
        
        function [A,b]=getLinEqsOnParams (obj)
            %[C,d]=getLinEqsOnParams() returns the linear equality 
            %constriants C*x= d.
            n=obj.nrOfParams;
            if size(obj.linEqParamConstrs,1)~=0
                
                A=obj.linEqParamConstrs(:,1:n);
                b=obj.linEqParamConstrs(:,n+1);
            else
                A=zeros(0,n);
                b=zeros(0,1);
            end
        end
        function [A,b]=getLinIneqsOnParams (obj)
            %[A,b]=getLinIneqsOnParams() returns the linear inequality 
            %constriants A*x<= b.
            n=obj.nrOfParams;
            if size(obj.linIneqParamConstrs,1)~=0
            A=obj.linIneqParamConstrs(:,1:n);
            b=obj.linIneqParamConstrs(:,n+1); 
            else
                A=zeros(0,n);
                b=zeros(0,1);
            end
        end

        
        function n=getNrNonlinEqConstrsOnParams (obj)
            %n=getNrNonlinEqConstrsOnParams() returns the number of non 
            %linear and non semi-infinite equality constraints on the 
            %parameters x.
            n=obj.nrOfNonlinParamEqConstrs;
        end
        function n=getNrNonlinIneqConstrsOnParams (obj)
            %n=getNrNonlinIneqConstrsOnParams() returns the number of non 
            %linear and non semi-infinite inequality constraints on the 
            %parameters x.
            n=obj.nrOfNonlinParamIneqConstrs;
        end
        
        function [lb,ub]=getBoundsOnParams(obj)
            %[lb,ub]=getBoundsOnParams() returns the lower and the
            %upper bounds on the parameters x. lb and ub are two row
            %vectors. If a parameter is unbounded -inf/inf is returned.
            lb=obj.boundsOnParams(1,:);
            ub=obj.boundsOnParams(2,:);
        end
    end
    
end

