classdef DesignEllipseWithRotMat<Design
    %D=DesignEllipseWithRotMat(n) creates for n=2 and n=3 an instance of a
    %design D(x) given by:
    %
    %       D(x)={ y in R^2 with:
    %               (y-xpos)^T*Rot(x)^T*A(x)*Rot(x)*(y-xpos) -1 <= 0}
    %       
    %   Where for n=2
    %       Rot(x)=Rot(x5) is a rotation matrix
    %       A(x)=[1/x3^2 , 0 ; 0 , 1/x4^2]
    %       xpos=[x1;x2]
    %
    %       Vol(x)=pi*x3*x4
    %
    %       x in X = { x in R^4 with
    %               10^-6 <= x3
    %               10^-6 <= x4}
    %
    %   Where for n=3
    %       Rot(x)=Rot(x7,x8,x9) is a rotation matrix
    %       A(x)=[1/x4^2 , 0 ,0 ; 0 , 1/x5^2 , 0 ; 0 , 0 , 1/x6^2]
    %       xpos=[x1 ; x2 ; x3]
    %
    %       Vol(x)=4/3*pi*x4*x5*x6
    %
    %       x in X = { x in R^4 with
    %               10^-6 <= x4
    %               10^-6 <= x5
    %               10^-6 <= x6}

    properties (GetAccess=protected)
       
        dim;                            
        nrOfParams;                     
        nrOfDesignConstrs=1;             
        initValsParams;             
        
        linEqParamConstrs=[];            
        linIneqParamConstrs=[];             
        boundsOnParams
        
        nrOfNonlinParamIneqConstrs=0;    
        nrOfNonlinParamEqConstrs=0;      
        
        %volume of the unit-sphere
        volsphere;                      
        
        
        
        
        
       
    end
    
    methods %constructor
            
        function p=DesignEllipseWithRotMat(dim,varargin)
            assert(dim==2||dim==3,'Das Design ist nur f�r 2 und 3 Dimensionen implementiert');

            %compute properties with dim  
            p.dim = dim;
            n=dim+(dim+1)*dim/2;
            p.nrOfParams=n;
            p.initValsParams=ones(1,n);

            %compute volume of unit-sphere
            if mod(dim,2)
                k=(dim-1)/2;
                p.volsphere=2*(2*pi)^k/prod(1:2:dim);
            else
                k=dim/2;
                p.volsphere=pi^k/factorial(k);
            end
            %constraints on parameters
            p.boundsOnParams=[-inf*ones(1,dim),1.e-6 *ones(1,dim),-inf*ones(1,2*dim-3);inf*ones(1,p.nrOfParams)];
        end
        
    end
    
    methods
        %compute rotation matrix
        function R=rotation(obj,x)
            if obj.dim==2
                R=[cos(x(5)),-sin(x(5));sin(x(5)),cos(x(5))];
            else
                R=[cos(x(7)),-sin(x(7)),0;sin(x(7)),cos(x(7)),0;0,0,1]*...
                    [cos(x(8)),0,-sin(x(8));0,1,0;sin(x(8)),0,cos(x(8))]*...
                    [1,0,0;0,cos(x(9)),-sin(x(9));0,sin(x(9)),cos(x(9))];  
            end
        end
        %auxiliary function for computing the derivative
        function DR=Drotation(obj,i,x) 
            if obj.dim==2
                DR=[-sin(x(5)),-cos(x(5));cos(x(5)),-sin(x(5))];
            else
                switch i
                    case 1
                        DR=[-sin(x(7)),-cos(x(7)),0;cos(x(7)),-sin(x(7)),0;0,0,0]*...
                            [cos(x(8)),0,-sin(x(8));0,1,0;sin(x(8)),0,cos(x(8))]*...
                            [1,0,0;0,cos(x(9)),-sin(x(9));0,sin(x(9)),cos(x(9))];    
                    case 2
                        DR=[cos(x(7)),-sin(x(7)),0;sin(x(7)),cos(x(7)),0;0,0,1]*...
                            [-sin(x(8)),0,-cos(x(8));0,0,0;cos(x(8)),0,-sin(x(8))]*...
                            [1,0,0;0,cos(x(9)),-sin(x(9));0,sin(x(9)),cos(x(9))]; 
                    case 3
                        DR=[cos(x(7)),-sin(x(7)),0;sin(x(7)),cos(x(7)),0;0,0,1]*...
                            [cos(x(8)),0,-sin(x(8));0,1,0;sin(x(8)),0,cos(x(8))]*...
                            [0,0,0;0,-sin(x(9)),-cos(x(9));0,cos(x(9)),-sin(x(9))]; 
                end
              
            end
        end
    end
    
    methods
        %volume
        function v=vol(obj,x)                    

            v=obj.volsphere*prod(x(obj.dim+1:obj.dim*2));
        end
        function grad_v=gradVol(obj,x)
           grad_v=zeros(obj.nrOfParams,1);
           n=obj.dim;
           v1=obj.volsphere*prod(x(n+1:n*2));
           grad_v(n+1:2*n)=v1./(x(n+1:2*n))';

           
        end
        
        %design functions
        function u=evalNonlinDescrFuns(obj,x,y)     
            R=obj.rotation(x);
            n=obj.dim;
            A=R*diag(1./x(n+1:2*n).^2)*R';
            
            xPos=x(1:n);
            u = (y-xPos)*A*(y'-xPos')-1;
        end
        function Dy_u=DyNonlinDescrFuns(obj,x,y)
            R=obj.rotation(x);
            n=obj.dim;
            A=R*diag(1./x(n+1:2*n).^2)*R';
            
            xPos=x(1:n);
            Dy_u = 2*(y-xPos)*A;
             
        end
        %interior point
        function y=getInteriorPoint(obj,x)       
            y=x(1:obj.dim);
        end
        function Dx_y=DxInnerPoint(obj,~)
            Dx_y=[eye(obj.dim),zeros(obj.dim,obj.nrOfParams)];
        end
        %bounding box
        function [lb,ub]=getBoundingBox(obj,x)   
            n=obj.dim;
            m=max(x(n+1:n*2));
            lb=x(1:n)-m;
            ub=x(1:n)+m;
        end
%         function [Dx_lb,Dx_ub]=DxBoundingBox(obj,x)
%             
%         end

    end
    %transformation    
    methods
        function y=evalTrafoFun(obj,x,z)
            R=obj.rotation(x);
            m=size(z,1);
            n=obj.dim;
            y=ones(m,1)*x(1:n)+z*diag(x(n+1:2*n))*R';
        end
        function z=evalInvTrafoFun(obj,x,y)
            R=obj.rotation(x);
            
            m=size(y,1);
            n=obj.dim;
            z=(y-ones(m,1)*x(1:n))*R*diag(x(n+1:2*n).^-1);
        end
        function Dx_y=DxTrafoFun(obj,x,z)
             m=size(z,1);
            o=ones(m,1);
            n=obj.dim;
            R=obj.rotation(x);
            
            if n==2
                Dx_y=zeros(2,5,m);
                Dx_y(1,1,:)=o;
                Dx_y(2,2,:)=o;
                DR=obj.Drotation(4,x);
                for i=1:m
                    
                    Dx_y(:,3:4,i)=R*diag(z(i,:));
                    Dx_y(:,5,i)=DR*(z(i,:)*diag(x(n+1:2*n)))';

                end
                
            else
                Dx_y=zeros(3,9,m);
                Dx_y(1,1,:)=o;
                Dx_y(2,2,:)=o;
                Dx_y(3,3,:)=o;
                DR1=obj.Drotation(1,x);
                DR2=obj.Drotation(2,x);
                DR3=obj.Drotation(3,x);
                for i=1:m
                    Dx_y(:,4:6,i)=R*diag(z(i,:));
                    
                    Dx_y(:,7,i)=DR1*(z(i,:)*diag(x(n+1:2*n)))';
                    Dx_y(:,8,i)=DR2*(z(i,:)*diag(x(n+1:2*n)))';
                    Dx_y(:,9,i)=DR3*(z(i,:)*diag(x(n+1:2*n)))';
                end
            end
        end
    end
    
    
        methods
        function []=plotDesign(obj,x)
            switch obj.dim
                case 2
                    t = 0:pi/180:2*pi;
                    o = ones(size(t));
                    R=[cos(x(5)),-sin(x(5));sin(x(5)),cos(x(5))];
                    %circle
                    y1_temp = cos(t);
                    y2_temp = sin(t);
                    %Scaling
                    y1_temp2=x(3)*y1_temp;
                    y2_temp2=x(4)*y2_temp;
                    %Rotation
                    y=x(1:2)'*o+R*[y1_temp2;y2_temp2];

                    fill(y(1,:),y(2,:),'green');
                    hold on;
                case 3
                    %circle
                    [y1t,y2t,y3t]=sphere();
                    %scaling
                    y1t=x(4)*y1t;
                    y2t=x(5)*y2t;
                    y3t=x(6)*y3t;

                    R=[cos(x(7)),-sin(x(7)),0;sin(x(7)),cos(x(7)),0;0,0,1]*...
                            [cos(x(8)),0,-sin(x(8));0,1,0;sin(x(8)),0,cos(x(8))]*...
                            [1,0,0;0,cos(x(9)),-sin(x(9));0,sin(x(9)),cos(x(9))];

                    %rotation and translation
                    y1=R(1,1)*y1t+R(1,2)*y2t+R(1,3)*y3t+x(1);
                    y2=R(2,1)*y1t+R(2,2)*y2t+R(2,3)*y3t+x(2);
                    y3=R(3,1)*y1t+R(3,2)*y2t+R(3,3)*y3t+x(3);
                    s = surfl(y1, y2, y3);
                    set(s, 'EdgeColor', 'none' , 'FaceColor', 'green', 'FaceAlpha', 0.75, 'FaceLighting', 'gouraud');
                    camlight;
                otherwise
                    warning('Can not plot the problem in dimension higher than 3')
            end            
        end
    end
    
end


